package sun.util.logging;

import sun.misc.JavaLangAccess;
import sun.misc.SharedSecrets;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.MessageFormat;
import java.io.PrintStream;
import java.util.Date;
import java.util.Arrays;
import java.util.HashMap;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Iterator;
import java.lang.ref.WeakReference;
import java.util.Map;

public class PlatformLogger
{
    private static final int OFF = Integer.MAX_VALUE;
    private static final int SEVERE = 1000;
    private static final int WARNING = 900;
    private static final int INFO = 800;
    private static final int CONFIG = 700;
    private static final int FINE = 500;
    private static final int FINER = 400;
    private static final int FINEST = 300;
    private static final int ALL = Integer.MIN_VALUE;
    private static final PlatformLogger.Level DEFAULT_LEVEL;
    private static boolean loggingEnabled;
    private static Map<String, WeakReference<PlatformLogger>> loggers;
    private volatile PlatformLogger.LoggerProxy loggerProxy;
    private volatile PlatformLogger.JavaLoggerProxy javaLoggerProxy;
    
    public static synchronized PlatformLogger getLogger(String s) {
        Object referent = null;
        WeakReference<Object> weakReference = (WeakReference)PlatformLogger.loggers.get(s);
        if (weakReference != null) {
            referent = (Object)weakReference.get();
        }
        if (referent == null) {
            referent = new PlatformLogger(s);
            PlatformLogger.loggers.put(s, new WeakReference(referent));
            return referent;
        }
        return referent;
    }
    
    public static synchronized void redirectPlatformLoggers() {
        if (PlatformLogger.loggingEnabled || !LoggingSupport.isAvailable()) {
            return;
        }
        PlatformLogger.loggingEnabled = true;
        Iterator<Map.Entry<String, WeakReference>> iterator = PlatformLogger.loggers.entrySet().iterator();
        while (iterator.hasNext()) {
            PlatformLogger platformLogger = (PlatformLogger)((WeakReference)((Map.Entry)iterator.next()).getValue()).get();
            if (platformLogger != null) {
                platformLogger.redirectToJavaLoggerProxy();
            }
            else {
                continue;
            }
        }
        return;
    }
    
    private void redirectToJavaLoggerProxy() {
        PlatformLogger.DefaultLoggerProxy defaultLoggerProxy = (PlatformLogger.DefaultLoggerProxy)PlatformLogger.DefaultLoggerProxy.class.cast(this.loggerProxy);
        PlatformLogger.JavaLoggerProxy javaLoggerProxy = new PlatformLogger.JavaLoggerProxy(defaultLoggerProxy.name, defaultLoggerProxy.level);
        this.javaLoggerProxy = javaLoggerProxy;
        this.loggerProxy = javaLoggerProxy;
    }
    
    private PlatformLogger(String s) {
        if (PlatformLogger.loggingEnabled) {
            PlatformLogger.JavaLoggerProxy javaLoggerProxy = new JavaLoggerProxy(s);
            this.javaLoggerProxy = javaLoggerProxy;
            this.loggerProxy = javaLoggerProxy;
        }
        else {
            this.loggerProxy = new DefaultLoggerProxy(s);
        }
    }
    
    public boolean isEnabled() {
        return this.loggerProxy.isEnabled();
    }
    
    public String getName() {
        return this.loggerProxy.name;
    }
    
    public boolean isLoggable(PlatformLogger.Level level) {
        if (level == null) {
            throw new NullPointerException();
        }
        PlatformLogger.JavaLoggerProxy javaLoggerProxy = this.javaLoggerProxy;
        return (javaLoggerProxy != null) ? javaLoggerProxy.isLoggable(level) : this.loggerProxy.isLoggable(level);
    }
    
    public PlatformLogger.Level level() {
        return this.loggerProxy.getLevel();
    }
    
    public void setLevel(PlatformLogger.Level level) {
        this.loggerProxy.setLevel(level);
    }
    
    public void severe(String s) {
        this.loggerProxy.doLog(PlatformLogger.Level.SEVERE, s);
    }
    
    public void severe(String s, Throwable t) {
        this.loggerProxy.doLog(PlatformLogger.Level.SEVERE, s, t);
    }
    
    public void severe(String s, Object... array) {
        this.loggerProxy.doLog(PlatformLogger.Level.SEVERE, s, array);
    }
    
    public void warning(String s) {
        this.loggerProxy.doLog(PlatformLogger.Level.WARNING, s);
    }
    
    public void warning(String s, Throwable t) {
        this.loggerProxy.doLog(PlatformLogger.Level.WARNING, s, t);
    }
    
    public void warning(String s, Object... array) {
        this.loggerProxy.doLog(PlatformLogger.Level.WARNING, s, array);
    }
    
    public void info(String s) {
        this.loggerProxy.doLog(PlatformLogger.Level.INFO, s);
    }
    
    public void info(String s, Throwable t) {
        this.loggerProxy.doLog(PlatformLogger.Level.INFO, s, t);
    }
    
    public void info(String s, Object... array) {
        this.loggerProxy.doLog(PlatformLogger.Level.INFO, s, array);
    }
    
    public void config(String s) {
        this.loggerProxy.doLog(PlatformLogger.Level.CONFIG, s);
    }
    
    public void config(String s, Throwable t) {
        this.loggerProxy.doLog(PlatformLogger.Level.CONFIG, s, t);
    }
    
    public void config(String s, Object... array) {
        this.loggerProxy.doLog(PlatformLogger.Level.CONFIG, s, array);
    }
    
    public void fine(String s) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINE, s);
    }
    
    public void fine(String s, Throwable t) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINE, s, t);
    }
    
    public void fine(String s, Object... array) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINE, s, array);
    }
    
    public void finer(String s) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINER, s);
    }
    
    public void finer(String s, Throwable t) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINER, s, t);
    }
    
    public void finer(String s, Object... array) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINER, s, array);
    }
    
    public void finest(String s) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINEST, s);
    }
    
    public void finest(String s, Throwable t) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINEST, s, t);
    }
    
    public void finest(String s, Object... array) {
        this.loggerProxy.doLog(PlatformLogger.Level.FINEST, s, array);
    }
    
    static {
        DEFAULT_LEVEL = PlatformLogger.Level.INFO;
        PlatformLogger.loggingEnabled = (boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                String property = System.getProperty("java.util.logging.config.class");
                String property2 = System.getProperty("java.util.logging.config.file");
                return Boolean.valueOf(property != null || property2 != null);
            }
        });
        try {
            Class.forName("sun.util.logging.PlatformLogger$DefaultLoggerProxy", false, PlatformLogger.class.getClassLoader());
            Class.forName("sun.util.logging.PlatformLogger$JavaLoggerProxy", false, PlatformLogger.class.getClassLoader());
        }
        catch (ClassNotFoundException cause) {
            throw new InternalError(cause);
        }
        PlatformLogger.loggers = new HashMap();
    }
    
    public enum Level
    {
        ALL, 
        FINEST, 
        FINER, 
        FINE, 
        CONFIG, 
        INFO, 
        WARNING, 
        SEVERE, 
        OFF;
        
        Object javaLevel;
        private static final int[] LEVEL_VALUES;
        
        public int intValue() {
            return PlatformLogger.Level.LEVEL_VALUES[this.ordinal()];
        }
        
        static PlatformLogger.Level valueOf(int key) {
            switch (key) {
                case 300: {
                    return PlatformLogger.Level.FINEST;
                }
                case 500: {
                    return PlatformLogger.Level.FINE;
                }
                case 400: {
                    return PlatformLogger.Level.FINER;
                }
                case 800: {
                    return PlatformLogger.Level.INFO;
                }
                case 900: {
                    return PlatformLogger.Level.WARNING;
                }
                case 700: {
                    return PlatformLogger.Level.CONFIG;
                }
                case 1000: {
                    return PlatformLogger.Level.SEVERE;
                }
                case Integer.MAX_VALUE: {
                    return PlatformLogger.Level.OFF;
                }
                case Integer.MIN_VALUE: {
                    return PlatformLogger.Level.ALL;
                }
                default: {
                    int binarySearch = Arrays.binarySearch(PlatformLogger.Level.LEVEL_VALUES, 0, PlatformLogger.Level.LEVEL_VALUES.length - 2, key);
                    return values()[(binarySearch >= 0) ? binarySearch : (-binarySearch - 1)];
                }
            }
        }
        
        static {
            LEVEL_VALUES = new int[] { Integer.MIN_VALUE, 300, 400, 500, 700, 800, 900, 1000, Integer.MAX_VALUE };
        }
    }
    
    private abstract static class LoggerProxy
    {
        final String name;
        
        protected LoggerProxy(String name) {
            this.name = name;
        }
        
        abstract boolean isEnabled();
        
        abstract PlatformLogger.Level getLevel();
        
        abstract void setLevel(PlatformLogger.Level p0);
        
        abstract void doLog(PlatformLogger.Level p0, String p1);
        
        abstract void doLog(PlatformLogger.Level p0, String p1, Throwable p2);
        
        abstract void doLog(PlatformLogger.Level p0, String p1, Object... p2);
        
        abstract boolean isLoggable(PlatformLogger.Level p0);
    }
    
    private static final class DefaultLoggerProxy extends PlatformLogger.LoggerProxy
    {
        volatile PlatformLogger.Level effectiveLevel;
        volatile PlatformLogger.Level level;
        private static final String formatString;
        private Date date;
        
        private static PrintStream outputStream() {
            return System.err;
        }
        
        DefaultLoggerProxy(String s) {
            super(s);
            this.date = new Date();
            this.effectiveLevel = this.deriveEffectiveLevel(null);
            this.level = null;
        }
        
        @Override
        boolean isEnabled() {
            return this.effectiveLevel != PlatformLogger.Level.OFF;
        }
        
        @Override
        PlatformLogger.Level getLevel() {
            return this.level;
        }
        
        @Override
        void setLevel(PlatformLogger.Level level) {
            if (this.level != level) {
                this.level = level;
                this.effectiveLevel = this.deriveEffectiveLevel(level);
            }
        }
        
        @Override
        void doLog(PlatformLogger.Level level, String s) {
            if (this.isLoggable(level)) {
                outputStream().print(this.format(level, s, null));
            }
        }
        
        @Override
        void doLog(PlatformLogger.Level level, String s, Throwable t) {
            if (this.isLoggable(level)) {
                outputStream().print(this.format(level, s, t));
            }
        }
        
        @Override
        void doLog(PlatformLogger.Level level, String s, Object... array) {
            if (this.isLoggable(level)) {
                outputStream().print(this.format(level, this.formatMessage(s, array), null));
            }
        }
        
        @Override
        boolean isLoggable(PlatformLogger.Level level) {
            PlatformLogger.Level effectiveLevel = this.effectiveLevel;
            return level.intValue() >= effectiveLevel.intValue() && effectiveLevel != PlatformLogger.Level.OFF;
        }
        
        private PlatformLogger.Level deriveEffectiveLevel(PlatformLogger.Level level) {
            return (level == null) ? PlatformLogger.DEFAULT_LEVEL : level;
        }
        
        private String formatMessage(String pattern, Object... arguments) {
            try {
                if (arguments == null || arguments.length == 0) {
                    return pattern;
                }
                if (pattern.indexOf("{0") >= 0 || pattern.indexOf("{1") >= 0 || pattern.indexOf("{2") >= 0 || pattern.indexOf("{3") >= 0) {
                    return MessageFormat.format(pattern, arguments);
                }
                return pattern;
            }
            catch (Exception ex) {
                return pattern;
            }
        }
        
        private synchronized String format(PlatformLogger.Level level, String s, Throwable t) {
            this.date.setTime(System.currentTimeMillis());
            String string = "";
            if (t != null) {
                StringWriter out = new StringWriter();
                PrintWriter s2 = new PrintWriter(out);
                s2.println();
                t.printStackTrace(s2);
                s2.close();
                string = out.toString();
            }
            return String.format(PlatformLogger.DefaultLoggerProxy.formatString, new Object[] { this.date, this.getCallerInfo(), this.name, level.name(), s, string });
        }
        
        private String getCallerInfo() {
            String str = null;
            String methodName = null;
            JavaLangAccess javaLangAccess = SharedSecrets.getJavaLangAccess();
            Throwable t = new Throwable();
            int stackTraceDepth = javaLangAccess.getStackTraceDepth(t);
            String s = "sun.util.logging.PlatformLogger";
            int n = 1;
            for (int i = 0; i < stackTraceDepth; ++i) {
                StackTraceElement stackTraceElement = javaLangAccess.getStackTraceElement(t, i);
                String className = stackTraceElement.getClassName();
                if (n != 0) {
                    if (className.equals(s)) {
                        n = 0;
                    }
                }
                else if (!className.equals(s)) {
                    str = className;
                    methodName = stackTraceElement.getMethodName();
                    break;
                }
            }
            if (str != null) {
                return str + " " + methodName;
            }
            return this.name;
        }
        
        static {
            formatString = LoggingSupport.getSimpleFormat(false);
        }
    }
    
    private static final class JavaLoggerProxy extends PlatformLogger.LoggerProxy
    {
        private final Object javaLogger;
        
        JavaLoggerProxy(String s) {
            this(s, null);
        }
        
        JavaLoggerProxy(String s, PlatformLogger.Level level) {
            super(s);
            this.javaLogger = LoggingSupport.getLogger(s);
            if (level != null) {
                LoggingSupport.setLevel(this.javaLogger, level.javaLevel);
            }
        }
        
        @Override
        void doLog(PlatformLogger.Level level, String s) {
            LoggingSupport.log(this.javaLogger, level.javaLevel, s);
        }
        
        @Override
        void doLog(PlatformLogger.Level level, String s, Throwable t) {
            LoggingSupport.log(this.javaLogger, level.javaLevel, s, t);
        }
        
        @Override
        void doLog(PlatformLogger.Level level, String s, Object... array) {
            if (!this.isLoggable(level)) {
                return;
            }
            int n = (array != null) ? array.length : 0;
            String[] array2 = new String[n];
            for (int i = 0; i < n; ++i) {
                array2[i] = String.valueOf(array[i]);
            }
            LoggingSupport.log(this.javaLogger, level.javaLevel, s, array2);
            return;
        }
        
        @Override
        boolean isEnabled() {
            return LoggingSupport.isLoggable(this.javaLogger, PlatformLogger.Level.OFF.javaLevel);
        }
        
        @Override
        PlatformLogger.Level getLevel() {
            Object level = LoggingSupport.getLevel(this.javaLogger);
            if (level == null) {
                return null;
            }
            try {
                return PlatformLogger.Level.valueOf(LoggingSupport.getLevelName(level));
            }
            catch (IllegalArgumentException ex) {
                return PlatformLogger.Level.valueOf(LoggingSupport.getLevelValue(level));
            }
        }
        
        @Override
        void setLevel(PlatformLogger.Level level) {
            LoggingSupport.setLevel(this.javaLogger, (level == null) ? null : level.javaLevel);
        }
        
        @Override
        boolean isLoggable(PlatformLogger.Level level) {
            return LoggingSupport.isLoggable(this.javaLogger, level.javaLevel);
        }
        
        static {
            for (PlatformLogger.Level level : PlatformLogger.Level.values()) {
                level.javaLevel = LoggingSupport.parseLevel(level.name());
            }
        }
    }
}
